% This file runs a few scenarios for a chosen test system to eveluate the
% error produced by the linear power flow (LPF) and the Multi-Region Linear
% Power Flow (MR-LPF) by comparing the results against the Newton-Raphson
% (NR) method. MATPOWER is employed as a reference to run the NR power flow.
% Further details about the LPF algorithm can be found in the following
% paper:
% J. R. Marti, H. Ahmadi, and L. Bashualdo, Linear power flow formulation
% based on a voltage-dependent load model, IEEE Trans. Power Del., vol. 28,
% no. 3, pp. 16821690, Jul. 2013.
% Further details about the MR-LPF algorithm can be found in the following
% paper:
% H. Ahmadi, J. R. Marti, and E. Vaahedi, The multi-region linear power 
% flow formulation, submitted to the IEEE Power Engineering Letters, 2015.
% -------------------------------------------------------------------------
% 
%                  Copyright (C) 2015  Hamed Ahmadi
% 
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% -------------------------------------------------------------------------
clc
clear
mpc = loadcase('node_70');
N = length(mpc.bus(:,1));
mpopt = mpoption('OUT_ALL',0);

CZ = -1*ones(N,1);CI = 2*ones(N,1);
PQ = mpc.bus(:,3:4);
Pmax = 4;
m = 15;
lambda = linspace(1,Pmax,m);
Err_LPF = zeros(N,m);
Err_MRLPF = zeros(N,m);
Vmin = zeros(m,1);
for k = 1:length(lambda)
    mpc.bus(:,3:4) = lambda(k)*PQ;
    r_NR = runpf(mpc,mpopt);
    r_LPF = LPF_modified(mpc,CZ,CI);
    r_MRLPF = MRLPF(mpc);
    Vmin(k) = min(r_NR.bus(:,8));
    Err_LPF(:,k) = 100*abs(r_NR.bus(:,8)-r_LPF(:,2))./r_NR.bus(:,8);
    Err_MRLPF(:,k) = 100*abs(r_NR.bus(:,8)-r_MRLPF(:,2))./r_NR.bus(:,8);
end
Err_LPF_max = max(Err_LPF);
Err_MRLPF_max = max(Err_MRLPF);
figure(1);semilogy(lambda,Err_LPF_max,'LineWidth',3);xlabel('\lambda');ylabel('Error(%)');title('LPF Error Analysis');grid on;
figure(2);semilogy(lambda,Err_MRLPF_max,'LineWidth',3);xlabel('\lambda');ylabel('Error(%)');title('MR-LPF Error Analysis');grid on;
figure(3);plot(lambda,Vmin,'LineWidth',3);xlabel('\lambda');ylabel('Minimum Voltage (p.u.)');title('Minimum Nodal Voltage');grid on;

