function r = LPF_modified(mpc,CZ,CI)
% This function runs the linear power flow (LPF) using the data structure
% given in mpc in MATPOWER format. CZ and CI are the load parameters,
% defining the loads' voltage dependence. For constant-power loads, use
% CZ = -1 and CI = 2. When the load parameters are different for P and Q, 
% use CZ and CI for P and CpZ and CpI for Q.
% Further details about the LPF algorithm can be found in the following
% paper:
% J. R. Marti, H. Ahmadi, and L. Bashualdo, Linear power flow formulation
% based on a voltage-dependent load model, IEEE Trans. Power Del., vol. 28,
% no. 3, pp. 16821690, Jul. 2013.
% 
% -------------------------------------------------------------------------
% 
%                  Copyright (C) 2015  Hamed Ahmadi
% 
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% -------------------------------------------------------------------------
n = length(mpc.bus(:,1));
% To form the admittance matrix, MATPOWER function is used.
y = makeYbus(mpc);
% It is assumed that the Node 1 is the substation.
Pl = mpc.bus(2:end,3)./mpc.baseMVA;Ql = mpc.bus(2:end,4)./mpc.baseMVA;
CZ = CZ(2:end);CI = CI(2:end);
% CpI and CpZ are for the reactive power component of loads.
CpI = CI;CpZ = CZ;
% Calculate the conductance and susceptance matrices
G = real(y);B = imag(y);
% Get rid of the first row and column associated with the source node
G1 = G(2:end,2:end);B1 = B(2:end,2:end);G2 = G1;B2 = B1;
% Add diagonal elements associated with the loads
G1 = G1 + diag(Pl.*CZ);
B1 = B1 - diag((CZ + CI).*Ql);
G2 = G2 + diag((CZ + CI).*Pl);
B2 = B2 - diag(Ql.*CpZ);
% Form the coefficients matrix and the left-hand side for the LPF. 
A = [G1 -B1;B2 G2];b = [-Pl.*CI - G(2:end,1);Ql.*CpI - B(2:end,1)];
% Solve the system of linear equations for Vb = [Vre;Vim]
Vb = A\b;
% Extract the real and imaginary parts of nodal voltages
VR = [1;Vb(1:n-1)];VI = [0;Vb(n:end)];
% Prepare the output: first column is the nodes
r(:,1) = mpc.bus(:,1);
% The second column contains the voltage magnitudes
r(:,2) = sqrt(VR.^2 + VI.^2);
% The third column contains the voltage angles in degrees
r(:,3) = 180/pi*atan2(VI,VR);
F = mpc.branch(:,1);T = mpc.branch(:,2);
% calculate power losses and total power delivered by the substation
PLoss = 1000*mpc.baseMVA*sum((mpc.branch(:,3)./(mpc.branch(:,3).^2+mpc.branch(:,4).^2)).*((VR(F)-VR(T)).^2 + (VI(F)-VI(T)).^2));
Psubstation = mpc.baseMVA*(G(1,:)*VR-B(1,:)*VI);
Qsubstation = -mpc.baseMVA*(G(1,:)*VI+B(1,:)*VR);
% Print the results
display('-----------------------------------------------------------')
display('              The Linear Power Flow Solution               ')
display('-----------------------------------------------------------')
display(['Total power losses:  ',num2str(PLoss),' kW'])
display(['Total active power delivered at the substation:  ',num2str(Psubstation),' MW'])
display(['Total reactive power delivered at the substation:  ',num2str(Qsubstation),' MVAR'])
display('-----------------------------------------------------------')
end